/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source$
 *
 *  $Revision$
 *
 *	$Log$
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002 by Michael Neuroth.										 *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

/*
  	Changes:

	23.8.2003	Menu item "Word wrap", "Overwrite" (toggle),
    24.8.2003	Save Script-Extention in config file, Tabspace handling implemented
	30.8.2003	saving the last selected directory implemented
	 1.4.2004	fixes for Zaurus c860 (layout manager), flag for show startup file dialog implemented
	30.4.2004,  porting QScintilla back to Qt 2.3.2 for embedded Qt
	 1.5.2004	using QScintilla for the script editor, many new editor features

	new features:
	  * syntax hightlighing
	  * find text
	  * replace text
	  * show line numbers
	  * autoindent
	  * tabs as tab or spaces
	  * show whitespaces
	  * Zoom in/out
	  * set ratio of output to script window in config dialog
	  * improve Import Dialog --> use layout manager, needed for c860
	  * bugfix: crash with popen and gnuplot? --> bugfix in buffer-length

	open:
	  * what about the space between lines? is it configurable in QScintilla?
	  (* redo/undo icons in der Toolleiste)
	  (* autocompletion)
	  (* add/remove marker, find/jump to marker)
	  (* mehrere Datein editieren --> tab-view)
*/

#include "visiscript.h"

#include "ofontselector.h"
#ifdef _with_opie_filedialog
#include "ofileselector.h"
#include "ofiledialog.h"
#else
#include "filedialogs.h"
#endif

#include "mymledit.h"
#include "codeedit.h"
#include "qextscintillalexercpp.h"
#include "qextscintillalexercsharp.h"
#include "qextscintillalexerpython.h"
#include "qextscintillalexerjava.h"
#include "qextscintillalexerjavascript.h"
#include "qextscintillalexeridl.h"
#include "qextscintillalexersql.h"
#include "qextscintillalexerhtml.h"
#include "FontDlg.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fstream>
#include <sys/wait.h>

using namespace std;

#include <qpe/qpeapplication.h>
#include <qpe/qpemenubar.h>
#include <qpe/qpetoolbar.h>
#include <qpe/resource.h>
#include <qpe/fileselector.h>
#include <qpe/config.h>

#include <qaction.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
//#include <qtextview.h>
#include <qfontdialog.h>
#include <qfont.h>
#include <qfontdatabase.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qwidgetstack.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qdialog.h>
#include <qmessagebox.h>
#include <qclipboard.h>

#include "configuredlgimpl.h"
#include "SaveAsDlg.h"
#include "AboutDlg.h"
#include "FindDlg.h"
#include "ReplaceDlg.h"

#define _TEMP_SCRIPT_FILE		"_tsf.xyz"
#define _TEMP_ERR_FILE			"_err.txt"
#define _TEMP_OUT_FILE			"_out.txt"
#define _VISISCRIPT_NAME		"visiscript"

const int c_iSmall = 8;
const int c_iMedium = 10;
const int c_iLarge = 12;

const int c_iMinOutput = 80;
const int c_iMaxOutput = 200;
const int c_iMinScript = 80;
const int c_iMaxScript = 40;

const int c_iMaxOutputRatio = 70;
const int c_iMinOutputRatio = 15;

#define LANG_NONE 		1
#define LANG_CPP		2
#define LANG_CSHARP		3
#define LANG_PYTHON		4
#define LANG_JAVA		5
#define LANG_JAVASCRIPT	6
#define LANG_MINSCRIPT	7
#define LANG_RUBY		8
#define LANG_PERL		9
#define LANG_PHP		10
#define LANG_IDL		11
#define LANG_HTML		12
#define LANG_SQL		13
#define LANG_MAX_ID		LANG_SQL+1

// ************************************************************************

static int u_id = 1;
static int get_unique_id()
{
    return u_id++;
}

static void copyDocLnk( const DocLnk & source, DocLnk & target )
{
	target.setName(source.name());
	target.setFile(source.file());
	target.setLinkFile(source.linkFile());
	target.setComment(source.comment());
	target.setType(source.type());
	target.setCategories(source.categories());
}

static unsigned long GetFileSize( const char *sFileName )
{
	fstream aFile( sFileName, ios::in );

	if( aFile.good() )
	{
		aFile.seekg( 0, ios::end );
		return (unsigned long)aFile.tellg();
	}
	return (unsigned long)-1;		// Datei nicht gefunden !
}

static bool ReadTextFile( const char * sFullFileName, QString & sText )
{
	fstream aFile( sFullFileName, ios::in );
	unsigned long nSize = GetFileSize( sFullFileName );

	if( aFile.good() )
	{
		char * sBuffer = new char [ nSize + 1 ];

		unsigned long i = 0;
		for( i=0; (i<nSize) && (!aFile.eof()); i++ )
		{
			aFile.get( sBuffer[i] );
		}
		sBuffer[ (nSize == i) ? nSize : i-1 ] = 0;
		sText = sBuffer;
		delete [] sBuffer;
		return true;
	}
	return false;
}

static bool WriteTextFile( const char * sFullFileName, const char * sText, bool bAppend = false )
{
	fstream aFile( sFullFileName, (bAppend ? ios::app : ios::out) );

	if( aFile.good() )
	{
		aFile << sText;
		return true;
	}
	return false;
}

QString GetExtentionFromScriptCmd( const QString & sCmd )
{
	QString sRet;

	if( sCmd == "minscript" )
	{
		sRet = "text/minscript";
	}
	else if( sCmd == "python" )
	{
		sRet = "text/python";
	}
	else if( sCmd == "perl" )
	{
		sRet = "text/perl";
	}
	else if( sCmd == "ruby" )
	{
		sRet = "text/ruby";
	}
	else
	{
		sRet = "text/plain";
	}

	return sRet;
}

// ************************************************************************
/*
 *  Constructs a VisiScript which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */
VisiScript::VisiScript( QWidget* parent,  const char* name, WFlags fl )
    : QMainWindow( parent, name, fl ),
	  m_pFileSelector( 0 ),
	  m_pScript( 0 ),
	  m_pLexer( 0 ),
	  m_pOutput( 0 ),
	  m_sScriptCmd( "minscript" ),
	  m_sScriptExtention( "text/plain" ),
	  m_sFontName( "fixed" ),
	  m_sFontStyle( "normal" ),
	  m_iFontSize( c_iMedium ),
	  m_bSaveOutput( false ),
	  m_iWordWrap( 0 ),
	  m_iOverwrite( 0 ),
	  m_iSpacesForTabs( 0 ),
	  m_iShowWhitespaces( 0 ),
	  m_iShowLineNumbers( 1 ),
	  m_iAutoIndent( 0 ),
	  m_iTabSpace( 4 ),
	  m_iLanguageID( LANG_MINSCRIPT ),
	  m_sExeclCmd( "/bin/bash" ),
	  m_iExecuteMode( EXECUTE_MODE_POPEN ),
	  m_iShowStartupFileDialog( 1 ),
	  m_iOutputRatio( 30 ),
	  m_bIsReplace( false ),
	  m_bRegularExpr( false ),
	  m_bCaseSensitive( false ),
	  m_bOnlyWholeWords( false ),
	  m_bSearchBackward( false ),
	  m_bReplaceAll( false )
{
	loadInit();

	QMultiLineEdit::setDefaultTabStop( m_iTabSpace );

    setToolBarsMovable( FALSE );

	// *************************************************************
	// *************************************************************

    m_pMenu = new QPEToolBar( this );
    m_pMenu->setHorizontalStretchable( TRUE );

    m_pMenuBar = new QPEMenuBar( m_pMenu );
    pFile = new QPopupMenu( this );
    pEdit = new QPopupMenu( this );
    pView = new QPopupMenu( this );
	pLanguages = new QPopupMenu( pView );

    connect( pFile, SIGNAL( aboutToShow() ), this, SLOT( sltFileAboutToShow() ) );
    connect( pEdit, SIGNAL( aboutToShow() ), this, SLOT( sltEditAboutToShow() ) );
    connect( pView, SIGNAL( aboutToShow() ), this, SLOT( sltViewAboutToShow() ) );
    connect( pLanguages, SIGNAL( aboutToShow() ), this, SLOT( sltLanguagesAboutToShow() ) );
    connect( pLanguages, SIGNAL( activated(int) ), this, SLOT( sltLanguageActivated(int) ) );

    m_pButtonBar = new QPEToolBar( this );

    m_pMenuBar->insertItem( tr( "File" ), pFile );
    m_pMenuBar->insertItem( tr( "Edit" ), pEdit );
    m_pMenuBar->insertItem( tr( "View" ), pView );

	// ++++++++++++++++++++++++++++++++++++++++++

    QAction * aAction = new QAction( tr( "New" ), Resource::loadPixmap( "new" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileNew() ) );
    //aAction->addTo( m_pButtonBar );
    aAction->addTo( pFile );

    aAction = new QAction( tr( "Open" ), Resource::loadPixmap( "fileopen" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileOpen() ) );
    //aAction->addTo( m_pButtonBar );
    aAction->addTo( pFile );

    aAction = new QAction( tr( "Save" ), /*Resource::loadPixmap( "filesave" ),*/ QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSave() ) );
    aAction->addTo( pFile );

    aAction = new QAction( tr( "Save as..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSaveAs() ) );
    aAction->addTo( pFile );

	pFile->insertSeparator();

    aAction = new QAction( tr( "Import..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileImport() ) );
    aAction->addTo( pFile );

    aAction = new QAction( tr( "Export..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileExport() ) );
    aAction->addTo( pFile );

    aAction = new QAction( tr( "Export output..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSaveOutputAs() ) );
    aAction->addTo( pFile );

	pFile->insertSeparator();

	// "settings"
    aAction = new QAction( tr( "About..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltShowAbout() ) );
    aAction->addTo( pFile );

	// ++++++++++++++++++++++++++++++++++++++++++

    aAction = new QAction( tr( "Undo" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditUndo() ) );
    aAction->addTo( pEdit );

    aAction = new QAction( tr( "Redo" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditRedo() ) );
    aAction->addTo( pEdit );

    pEdit->insertSeparator();

	m_pCutAction = new QAction( tr( "Cut" ), QString::null, 0, this, 0 );
    connect( m_pCutAction, SIGNAL( activated() ), this, SLOT( sltEditCut() ) );
    m_pCutAction->addTo( pEdit );

    m_pCopyAction = new QAction( tr( "Copy" ), QString::null, 0, this, 0 );
    connect( m_pCopyAction, SIGNAL( activated() ), this, SLOT( sltEditCopy() ) );
    m_pCopyAction->addTo( pEdit );

    m_pPasteAction = new QAction( tr( "Paste" ), QString::null, 0, this, 0 );
    connect( m_pPasteAction, SIGNAL( activated() ), this, SLOT( sltEditPaste() ) );
    m_pPasteAction->addTo( pEdit );

    pEdit->insertSeparator();

    aAction = new QAction( tr( "Find..." ), Resource::loadPixmap( "find" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditFindText() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( pEdit );

    aAction = new QAction( tr( "Find next" ), Resource::loadPixmap( "forward" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditFindNext() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( pEdit );

    aAction = new QAction( tr( "Replace..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditReplaceText() ) );
    aAction->addTo( pEdit );

    pEdit->insertSeparator();

    aAction = new QAction( tr( "Clear script" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltClearScript() ) );
    aAction->addTo( pEdit );

    aAction = new QAction( tr( "Clear output" ), Resource::loadPixmap( "editdelete" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltClearOutput() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( pEdit );

	// ++++++++++++++++++++++++++++++++++++++++++

    pEdit->insertSeparator();

    aAction = new QAction( tr( "Run" ), Resource::loadPixmap( /*"exec"*/"play" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltRun() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( pEdit );

	// ++++++++++++++++++++++++++++++++++++++++++
/*
	m_pFixedAction = new QAction( tr( "Fixed" ), QString::null, 0, this, 0, TRUE );
    connect( m_pFixedAction, SIGNAL( activated() ), this, SLOT( sltSelectFixed() ) );
    m_pFixedAction->addTo( pView );
	m_pHelveticaAction = new QAction( tr( "Helvetica" ), QString::null, 0, this, 0, TRUE );
    connect( m_pHelveticaAction, SIGNAL( activated() ), this, SLOT( sltSelectHelvetica() ) );
    m_pHelveticaAction->addTo( pView );
	m_pUnifontAction = new QAction( tr( "Unifont" ), QString::null, 0, this, 0, TRUE );
    connect( m_pUnifontAction, SIGNAL( activated() ), this, SLOT( sltSelectUnifont() ) );
    m_pUnifontAction->addTo( pView );

    pView->insertSeparator();

	m_pLargeAction = new QAction( tr( "Large" ), QString::null, 0, this, 0, TRUE );
    connect( m_pLargeAction, SIGNAL( activated() ), this, SLOT( sltSelectLarge() ) );
    m_pLargeAction->addTo( pView );
	m_pMediumAction = new QAction( tr( "Medium" ), QString::null, 0, this, 0, TRUE );
    connect( m_pMediumAction, SIGNAL( activated() ), this, SLOT( sltSelectMedium() ) );
    m_pMediumAction->addTo( pView );
	m_pSmallAction = new QAction( tr( "Small" ), QString::null, 0, this, 0, TRUE );
    connect( m_pSmallAction, SIGNAL( activated() ), this, SLOT( sltSelectSmall() ) );
    m_pSmallAction->addTo( pView );
*/
    aAction = new QAction( tr( "Font..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltSelectFont() ) );
    aAction->addTo( pView );
/*
    pView->insertSeparator();

	aAction = new QAction( tr( "Zoom in" ), QString::null, 0, this, 0, FALSE );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomIn() ) );
    aAction->addTo( pView );
	aAction = new QAction( tr( "Zoom out" ), QString::null, 0, this, 0, FALSE );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomOut() ) );
    aAction->addTo( pView );
*/
	// "settings"
    aAction = new QAction( tr( "Configuration..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltConfiguration() ) );
    aAction->addTo( pView );

	pView->insertItem( tr("Language"), pLanguages );
	pLanguages->insertItem( tr("none"), LANG_NONE );
	pLanguages->insertItem( tr("minscript"), LANG_MINSCRIPT );
	pLanguages->insertItem( tr("C++"), LANG_CPP );
	pLanguages->insertItem( tr("C#"), LANG_CSHARP );
	pLanguages->insertItem( tr("Python"), LANG_PYTHON );
	pLanguages->insertItem( tr("Java"), LANG_JAVA );
	pLanguages->insertItem( tr("JavaScript"), LANG_JAVASCRIPT );
		pLanguages->insertItem( tr("Ruby"), LANG_RUBY );
		pLanguages->insertItem( tr("Perl"), LANG_PERL );
		pLanguages->insertItem( tr("PHP"), LANG_PHP );
	pLanguages->insertItem( tr("IDL"), LANG_IDL );
	pLanguages->insertItem( tr("HTML"), LANG_HTML );
	pLanguages->insertItem( tr("SQL"), LANG_SQL );

	pView->insertSeparator();

    aAction = new QAction( tr( "Zoom in" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomIn() ) );
    aAction->addTo( pView );
    aAction = new QAction( tr( "Zoom out" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomOut() ) );
    aAction->addTo( pView );

	pView->insertSeparator();

	m_pMaximizeScript = new QAction( tr( "Maximize script" ), QString::null, 0, this, 0, TRUE );
    connect( m_pMaximizeScript, SIGNAL( activated() ), this, SLOT( sltMaximizeScript() ) );
    m_pMaximizeScript->addTo( pView );
    m_pMaximizeOutput = new QAction( tr( "Maximize output" ), QString::null, 0, this, 0, TRUE );
    connect( m_pMaximizeOutput, SIGNAL( activated() ), this, SLOT( sltMaximizeOutput() ) );
    m_pMaximizeOutput->addTo( pView );

	pView->insertSeparator();

	m_pToggleWordWrap = new QAction( tr( "Word wrap" ), QString::null, 0, this, 0, TRUE );
    connect( m_pToggleWordWrap, SIGNAL( activated() ), this, SLOT( sltToggleWordWrap() ) );
    m_pToggleWordWrap->addTo( pView );
    m_pToggleOverwrite = new QAction( tr( "Overwrite" ), QString::null, 0, this, 0, TRUE );
    connect( m_pToggleOverwrite, SIGNAL( activated() ), this, SLOT( sltToggleOverwrite() ) );
    m_pToggleOverwrite->addTo( pView );


	// *************************************************************
	// *************************************************************

	m_pStack = new QWidgetStack( this );
    setCentralWidget( m_pStack );

    m_pVBox = new QVBox( m_pStack );
	m_pVBox->setSpacing( 4 );
	m_pVBox->setMargin( 2 );

    m_pScript = new MyQCodeEdit( m_pVBox, "m_pScript" );
	m_pScript->setTabWidth( m_iTabSpace );
	m_pScript->setLexer( 0 );
	m_pScript->setMarginLineNumbers( 1, true );
	m_pScript->setOverwriteMode( (bool)m_iOverwrite );
//m_pScript = new QTextView( m_pVBox, "m_pScript" );
//(min)	m_pScript->setUndoEnabled( true );
//(min)	m_pScript->setUndoDepth( 10 );

    m_pOutput = new MyQMultiLineEdit( m_pVBox, "m_pOutput" );
    m_pOutput->setReadOnly( TRUE );

	// ** script edit field should be larger than the output edit field
    m_pVBox->setStretchFactor( m_pScript, 3 );
	//sltMaximizeOutput();

	m_pStack->addWidget( m_pVBox, get_unique_id() );

	updateFileSelector();

	if( m_iShowStartupFileDialog )
	{
	    sltFileOpen();
	}
	else
	{
		sltShowScript();
		updateCaption();
	}

	// font settings updaten
	UpdateFont();
	UpdateFlags();
	UpdateLanguage();
	//m_pFixedAction->setOn( true );
	//m_pMediumAction->setOn( true );

#ifndef _with_opie_filedialog
	m_pExportFileDlg = new FileDialogS(this, "save",TRUE, WType_Modal, TRUE);
	connect( m_pExportFileDlg, SIGNAL( doneLoad() ), this, SLOT( sltDoImport() ) );
    connect( m_pExportFileDlg, SIGNAL( done() ), this, SLOT( sltDoExport() ) );
	m_pExportFileDlg->adjustSize();
#endif
}

/*
 *  Destroys the object and frees any allocated resources
 */
VisiScript::~VisiScript()
{
	m_pScript->setLexer( 0 );
	delete m_pLexer;

#ifndef _with_opie_filedialog
	delete m_pExportFileDlg;
#endif
	// no need to delete child widgets, Qt does it all for us
}

void VisiScript::sltFileNew()
{
	CheckData();

	sltNewFile( DocLnk() );
}

void VisiScript::updateFileSelector()
{
    delete m_pFileSelector;

    m_pFileSelector = new FileSelector( m_sScriptExtention, m_pStack, "fileselector" , TRUE, FALSE );
    connect( m_pFileSelector, SIGNAL( closeMe() ), this, SLOT( sltShowScript() ) );
    connect( m_pFileSelector, SIGNAL( newSelected( const DocLnk &) ), this, SLOT( sltNewFile( const DocLnk & ) ) );
    connect( m_pFileSelector, SIGNAL( fileSelected( const DocLnk &) ), this, SLOT( sltOpenFile( const DocLnk & ) ) );
}

void VisiScript::sltFileOpen()
{
    CheckData();

    m_pMenu->hide();
    m_pButtonBar->hide();
    clearWState (WState_Reserved1 );
    updateFileSelector();
    m_pStack->raiseWidget( m_pFileSelector );
    m_pFileSelector->reread();
    updateCaption();
}

void VisiScript::sltFileSave()
{
    FileManager fm;

	QString s = m_pScript->text();
    if( fm.saveFile( m_aDoc, s ) )
	{
		m_pScript->setEdited( false );
    }
	else
	{
		// ggf. todo --> fehlermeldung ausgeben !
	}
}

void VisiScript::sltFileSaveAs()
{
	SaveAsDlg * pDlg = new SaveAsDlg( this, "saveasdlg", /*bModal*/TRUE );

	pDlg->m_pFileName->setText( "empty" );

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		// ok, pressed
		QString sFileName = pDlg->m_pFileName->text();

	    FileManager fm;

		QString s = m_pScript->text();
		QString sExt = pDlg->m_pType->currentText();

		// create a new Doc-Link for the copy of the actual file
		DocLnk aDoc;
		aDoc.setType( /*"text/plain"*/sExt );
		aDoc.setName( sFileName );
	    if( fm.saveFile( aDoc, s ) )
		{
			// schreiben war ok
			m_pScript->setEdited( false );
			copyDocLnk( aDoc, m_aDoc );
			updateCaption( m_aDoc.name() );
		}
		else
		{
			// ggf. todo --> fehlermeldung ausgeben !
		}
	}

	delete pDlg;
}

void VisiScript::sltFileImport()
{
#ifdef _with_opie_filedialog
	CheckData();

	QString sFileName = OFileDialog::getOpenFileName( OFileSelector::NORMAL );

	if( !sFileName.isNull() )
	{
		QString sText;
		bool bOk = ReadTextFile( sFileName.latin1(), sText );

		if( bOk )
		{
	    	m_pScript->setText( sText );
    		m_pScript->setEdited( FALSE );
			updateCaption( sFileName+" #" );

			// update/clear doc-link !
			DocLnk aDoc;
			//aDoc.setType( "text/plain" );
			//aDoc.setName( *pFileName );
			copyDocLnk( aDoc, m_aDoc );
		}
	}
#else
	m_pExportFileDlg->setLoad( TRUE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
}

void VisiScript::sltFileExport()
{
#ifdef _with_opie_filedialog
	QString sFileName = OFileDialog::getSaveFileName( OFileSelector::NORMAL );
	//cout << "Export: " << (const char *)sFileName << endl;

	if( !sFileName.isNull() )
	{
		QString sText = m_pScript->text();
		bool bOk = WriteTextFile( sFileName.latin1(), sText );

		if( bOk )
		{
    		m_pScript->setEdited( FALSE );
			updateCaption( sFileName+" #" );
		}
	}
#else
	m_bSaveOutput = false;
	m_pExportFileDlg->setLoad( FALSE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
}

void VisiScript::sltFileSaveOutputAs()
{
#ifdef _with_opie_filedialog
	QString sFileName = OFileDialog::getSaveFileName( OFileSelector::NORMAL );
	//cout << "SaveOutput: " << (const char *)sFileName << endl;

	if( !sFileName.isNull() )
	{
		QString sText = m_pOutput->text();
		/*bool bOk =*/ WriteTextFile( sFileName.latin1(), sText );
	}
#else
	m_bSaveOutput = true;
	m_pExportFileDlg->setLoad( FALSE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
}

#ifndef _with_opie_filedialog
void VisiScript::sltDoImport()
{
	CheckData();

	QString * pFileName = m_pExportFileDlg->getAbsSelectedFileName();

	m_sLastDirectory = m_pExportFileDlg->getDirectory();

	QString sText;
	bool bOk = ReadTextFile( (const char *)pFileName->latin1(), sText );

	if( bOk )
	{
	    m_pScript->setText( sText );
	 	m_pScript->setEdited(FALSE);
		updateCaption( *pFileName+" #" );

		// update/clear doc-link !
		DocLnk aDoc;
		//aDoc.setType( "text/plain" );
		//aDoc.setName( *pFileName );
		copyDocLnk( aDoc, m_aDoc );
	}

	delete pFileName;
}

void VisiScript::sltDoExport()
{
	QString * pFileName = m_pExportFileDlg->getAbsSelectedFileName();

	m_sLastDirectory = m_pExportFileDlg->getDirectory();

	QString sText;
	if( m_bSaveOutput )
	{
		sText = m_pOutput->text();
	}
	else
	{
		sText = m_pScript->text();
	}

	bool bOk = WriteTextFile( (const char *)pFileName->latin1(), sText );

	if( bOk && !m_bSaveOutput )
	{
    	m_pScript->setEdited(FALSE);
		updateCaption( *pFileName+" #" );
	}

	delete pFileName;
}
#endif

static void SetComboBoxToText( QComboBox * pCombo, const QString & sText )
{
	for( int i=0; i<pCombo->count(); i++ )
	{
		if( pCombo->text( i ) == sText )
		{
			pCombo->setCurrentItem( i );
			break;
		}
	}
}

void VisiScript::sltConfiguration()
{
	ConfigureDlgImpl * pDlg = new ConfigureDlgImpl( this, "configuredlg", /*bModal*/TRUE );

	pDlg->adjustSize();

	pDlg->m_pScriptCmd->setText( m_sScriptCmd );

	SetComboBoxToText( pDlg->m_pScriptCmdList, m_sScriptCmd );
	SetComboBoxToText( pDlg->m_pType, m_sScriptExtention );

	pDlg->m_pTabSpace->setText( QString::number( m_iTabSpace ) );
	pDlg->m_pSpacesForTabs->setChecked( (bool)m_iSpacesForTabs );
	pDlg->m_pShowWhitespaces->setChecked( (bool)m_iShowWhitespaces );
	pDlg->m_pShowLineNumbers->setChecked( (bool)m_iShowLineNumbers );
	pDlg->m_pAutoIndent->setChecked( (bool)m_iAutoIndent );
	pDlg->m_pOutputRatio->setValue( m_iOutputRatio );

	pDlg->m_pExecuteCmd->setText( m_sExeclCmd );

	if( m_iExecuteMode == EXECUTE_MODE_POPEN )
	{
		pDlg->m_pUsepopen->setChecked( TRUE );
	}
	if( m_iExecuteMode == EXECUTE_MODE_EXECL )
	{
		pDlg->m_pUseexecl->setChecked( TRUE );
	}
	if( m_iShowStartupFileDialog==0 )
	{
		pDlg->m_pShowFileDialog->setChecked( FALSE );
	}
	else
	{
		pDlg->m_pShowFileDialog->setChecked( TRUE );
	}

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		// ok, pressed
		m_sScriptCmd = pDlg->m_pScriptCmd->text();

		m_sScriptExtention = pDlg->m_pType->currentText();

		m_iSpacesForTabs = (int)pDlg->m_pSpacesForTabs->isChecked();
		m_iShowWhitespaces = (int)pDlg->m_pShowWhitespaces->isChecked();
		m_iShowLineNumbers = (int)pDlg->m_pShowLineNumbers->isChecked();
		m_iAutoIndent = (int)pDlg->m_pAutoIndent->isChecked();
		m_iOutputRatio = pDlg->m_pOutputRatio->value();

		m_iTabSpace = pDlg->m_pTabSpace->text().toInt();
		QMultiLineEdit::setDefaultTabStop( m_iTabSpace );
		m_pScript->setTabWidth( m_iTabSpace );
		m_pScript->repaint( TRUE );

		m_sExeclCmd = pDlg->m_pExecuteCmd->text();

		if( pDlg->m_pUsepopen->isChecked() )
		{
			m_iExecuteMode = EXECUTE_MODE_POPEN;
		}
		if( pDlg->m_pUseexecl->isChecked() )
		{
			m_iExecuteMode = EXECUTE_MODE_EXECL;
		}
		if( pDlg->m_pShowFileDialog->isChecked() )
		{
			m_iShowStartupFileDialog = 1;
		}
		else
		{
			m_iShowStartupFileDialog = 0;
		}

//m_pScript->setHorizScrollbar( false );
//m_pScript->setMaxWidth( 30 );

		UpdateFlags();
		UpdateScriptOutputRatio();

		saveInit();
	}
	delete pDlg;
}

void VisiScript::sltShowAbout()
{
	AboutDlg * pDlg = new AboutDlg( this, "aboutdlg", /*bModal*/TRUE );

	pDlg->adjustSize();

	/*int iRet =*/ pDlg->exec();

	delete pDlg;
}

void VisiScript::sltSelectFont()
{
#ifndef QT_NO_FONTDIALOG
	bool bOk = false;
	QFont aFont;

	aFont = QFontDialog::getFont( &bOk );
#else
	FontDlg * pDlg = new FontDlg( this, "fontdlg", /*bModal*/TRUE );

	pDlg->m_pFontWidget->setSelectedFont( m_sFontName, m_sFontStyle, m_iFontSize );

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_sFontName = pDlg->m_pFontWidget->fontFamily();
		m_sFontStyle = pDlg->m_pFontWidget->fontStyle();
		m_iFontSize = pDlg->m_pFontWidget->fontSize();
		UpdateFont();
		saveInit();
	}

	delete pDlg;
#endif
}
/*
void VisiScript::sltSelectHelvetica()
{
	m_pFixedAction->setOn( false );
	m_pUnifontAction->setOn( false );
	m_sFontName = "Helvetica";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectFixed()
{
	m_pHelveticaAction->setOn( false );
	m_pUnifontAction->setOn( false );
	m_sFontName = "fixed";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectUnifont()
{
	m_pFixedAction->setOn( false );
	m_pHelveticaAction->setOn( false );
	m_sFontName = "Unifont";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectSmall()
{
	m_pMediumAction->setOn( false );
	m_pLargeAction->setOn( false );
	m_iFontSize = c_iSmall;
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectMedium()
{
	m_pSmallAction->setOn( false );
	m_pLargeAction->setOn( false );
	m_iFontSize = c_iMedium;
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectLarge()
{
	m_pMediumAction->setOn( false );
	m_pSmallAction->setOn( false );
	m_iFontSize = c_iLarge;
	UpdateFont();
	saveInit();
}
*/

void VisiScript::sltZoomIn()
{
//	m_pScript->zoomIn();
	m_iFontSize++;
	UpdateFont();
	saveInit();
}

void VisiScript::sltZoomOut()
{
//	m_pScript->zoomOut();
	m_iFontSize--;
	UpdateFont();
	saveInit();
}

void VisiScript::sltMaximizeScript()
{
	if( !m_pMaximizeScript->isOn() )
	{
		m_pOutput->setMinimumSize( m_pOutput->minimumSize().width(), /*c_iMinScript*/GetHeightForOutput( m_iOutputRatio ) );
		m_pOutput->setMaximumSize( m_pOutput->maximumSize().width(), /*c_iMinScript*/GetHeightForOutput( m_iOutputRatio ) );
	}
	else
	{
		m_pMaximizeOutput->setOn( false );
		m_pOutput->setMinimumSize( m_pOutput->minimumSize().width(), /*c_iMaxScript*/GetHeightForOutput( c_iMinOutputRatio ) );
		m_pOutput->setMaximumSize( m_pOutput->maximumSize().width(), /*c_iMaxScript*/GetHeightForOutput( c_iMinOutputRatio ) );
	}
}

void VisiScript::sltMaximizeOutput()
{
	if( !m_pMaximizeOutput->isOn() )
	{
		m_pOutput->setMinimumSize( m_pOutput->minimumSize().width(), /*c_iMinOutput*/GetHeightForOutput( m_iOutputRatio ) );
		m_pOutput->setMaximumSize( m_pOutput->maximumSize().width(), /*c_iMinOutput*/GetHeightForOutput( m_iOutputRatio ) );
	}
	else
	{
		m_pMaximizeScript->setOn( false );
		m_pOutput->setMinimumSize( m_pOutput->minimumSize().width(), /*c_iMaxOutput*/GetHeightForOutput( c_iMaxOutputRatio ) );
		m_pOutput->setMaximumSize( m_pOutput->maximumSize().width(), /*c_iMaxOutput*/GetHeightForOutput( c_iMaxOutputRatio ) );
	}
}

void VisiScript::sltToggleWordWrap()
{
	if( !m_pToggleWordWrap->isOn() )
	{
//(min)		m_pScript->setWordWrap( QMultiLineEdit::NoWrap );
		m_pScript->setWordWrap( false );
		m_iWordWrap = 0;
	}
	else
	{
//(min)		m_pScript->setWordWrap( QMultiLineEdit::WidgetWidth );
		m_pScript->setWordWrap( true );
		m_iWordWrap = 1;
	}
}

void VisiScript::sltToggleOverwrite()
{
	if( !m_pToggleOverwrite->isOn() )
	{
		m_pScript->setOverwriteMode( false );
//		m_iOverwrite = 0;
	}
	else
	{
		m_pScript->setOverwriteMode( true );
//		m_iOverwrite = 1;
	}
}

void VisiScript::UpdateFont()
{
	QFontDatabase aFontDB;
	int iWeight = aFontDB.weight( m_sFontName, m_sFontStyle );
	bool bItalic = aFontDB.italic( m_sFontName, m_sFontStyle );
	QFont aTempFont( m_sFontName, m_iFontSize, iWeight, bItalic );
	if( m_pScript )
	{
		m_pScript->setFont( aTempFont );
	}
	if( m_pLexer )
	{
		m_pLexer->setFont( aTempFont );
		m_pLexer->setDefaultFont( aTempFont );
		m_pScript->setLexer( m_pLexer );
	}
	if( m_pOutput )
	{
		m_pOutput->setFont( aTempFont );
	}
}

void VisiScript::UpdateFlags()
{
	// restore values form the init/configuration file
	if( m_iWordWrap==0 )
	{
		m_pToggleWordWrap->setOn( false );
//(min)		m_pScript->setWordWrap( QMultiLineEdit::NoWrap );
		m_pScript->setWordWrap( false );
	}
	else
	{
		m_pToggleWordWrap->setOn( true );
//(min)		m_pScript->setWordWrap( QMultiLineEdit::WidgetWidth );
		m_pScript->setWordWrap( true );
	}
	if( !m_pScript->overwriteMode()/*m_iOverwrite==0*/ )
	{
		m_pToggleOverwrite->setOn( false );
		m_pScript->setOverwriteMode( false );
	}
	else
	{
		m_pToggleOverwrite->setOn( true );
		m_pScript->setOverwriteMode( true );
	}
	if( m_iSpacesForTabs==0 )
	{
		m_pScript->setIndentationsUseTabs( true );
	}
	else
	{
		m_pScript->setIndentationsUseTabs( false );
	}
	if( m_iShowWhitespaces==0 )
	{
		m_pScript->setWhitespaceVisibility( QextScintilla::WsInvisible );
		m_pScript->setEolVisibility( false );
	}
	else
	{
		m_pScript->setWhitespaceVisibility( QextScintilla::WsVisible );
		m_pScript->setEolVisibility( true );
	}
	if( m_iShowLineNumbers==0 )
	{
		m_pScript->setMarginLineNumbers( 1, true );
		m_pScript->setMarginWidth( 1, 0 );
	}
	else
	{
		m_pScript->setMarginLineNumbers( 1, true );
		m_pScript->setMarginWidth( 1, QString("9999") );
	}
	if( m_iAutoIndent==0 )
	{
		m_pScript->setAutoIndent( false );
	}
	else
	{
		m_pScript->setAutoIndent( true );
	}
}

void VisiScript::UpdateLanguage()
{
	if( m_pLexer )
	{
		m_pScript->setLexer( 0 );
		delete m_pLexer;
		m_pLexer = 0;
	}

	switch( m_iLanguageID )
	{
		case LANG_NONE:
			m_pLexer = 0;
			break;
		case LANG_MINSCRIPT:
		case LANG_CPP:
			m_pLexer = new QextScintillaLexerCPP();
			break;
		case LANG_CSHARP:
			m_pLexer = new QextScintillaLexerCSharp();
			break;
		case LANG_PYTHON:
			m_pLexer = new QextScintillaLexerPython();
			break;
		case LANG_JAVA:
			m_pLexer = new QextScintillaLexerJava();
			break;
		case LANG_JAVASCRIPT:
			m_pLexer = new QextScintillaLexerJavaScript();
			break;
		case LANG_RUBY:
		case LANG_PERL:
		case LANG_PHP:
			// vorlaeufig...
			m_pLexer = new QextScintillaLexerCPP();
			break;
		case LANG_IDL:
			m_pLexer = new QextScintillaLexerIDL();
			break;
		case LANG_HTML:
			m_pLexer = new QextScintillaLexerHTML();
			break;
		case LANG_SQL:
			m_pLexer = new QextScintillaLexerSQL();
			break;
		default:
			m_pLexer = 0;
	}

	UpdateFont();

	m_pScript->setLexer( m_pLexer );

	//m_pScript->setText( m_pScript->text() );

	m_pScript->update();
	m_pScript->repaint();
}

void VisiScript::UpdateLanguageMenu()
{
	// deselect all languages
	for( int i=LANG_NONE; i<LANG_MAX_ID; i++ )
	{
		pLanguages->setItemChecked( i, false );
	}
	// and select the new language
	pLanguages->setItemChecked( m_iLanguageID, true );
}

void VisiScript::UpdateScriptOutputRatio()
{
	sltMaximizeOutput();
}

int VisiScript::GetHeightForOutput( int iPercent )
{
	int iValue = height()*iPercent/100;
	return iValue;
}

void VisiScript::sltEditUndo()
{
	m_pScript->undo();
}

void VisiScript::sltEditRedo()
{
	m_pScript->redo();
}

void VisiScript::sltEditCut()
{
	if( m_pScript->hasMarkedTxt() )
	{
		m_pScript->cut();
	}
}

void VisiScript::sltEditCopy()
{
	if( m_pScript->hasMarkedTxt() )
	{
		m_pScript->copy();
	}
	else if( m_pOutput->hasMarkedTxt() )
	{
		m_pOutput->copy();
	}
}

void VisiScript::sltEditPaste()
{
	m_pScript->paste();
}

void VisiScript::sltClearOutput()
{
	m_pOutput->setText( "" );
}

void VisiScript::sltEditFindText()
{
	FindDlg * pDlg = new FindDlg( this, "findtextdlg", /*bModal*/TRUE );;

	pDlg->adjustSize();

	// if any text is marked, use this text for searching
	if( m_pScript->hasMarkedTxt() )
	{
		pDlg->m_pFindText->setText( m_pScript->markedTxt() );
	}
	else
	{
		pDlg->m_pFindText->setText( m_sFindText );
	}
	pDlg->m_pRegularExpr->setChecked( m_bRegularExpr );
	pDlg->m_pCaseSensitive->setChecked( m_bCaseSensitive );
	pDlg->m_pOnlyWholeWord->setChecked( m_bOnlyWholeWords );
	pDlg->m_pSearchBackward->setChecked( m_bSearchBackward );

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_bIsReplace = false;

		m_sFindText = pDlg->m_pFindText->text();
		m_bRegularExpr = pDlg->m_pRegularExpr->isChecked();
		m_bCaseSensitive = pDlg->m_pCaseSensitive->isChecked();
		m_bOnlyWholeWords = pDlg->m_pOnlyWholeWord->isChecked();
		m_bSearchBackward = pDlg->m_pSearchBackward->isChecked();

		m_pScript->findFirst( m_sFindText,
							  m_bRegularExpr,
		 					  m_bCaseSensitive,
							  m_bOnlyWholeWords,
							  !m_bSearchBackward );
	}

	delete pDlg;
}

void VisiScript::sltEditFindNext()
{
	bool bFound = m_pScript->findNext();

	if( m_bIsReplace && bFound )
	{
		m_pScript->replace( m_sReplaceText );
	}
}

void VisiScript::sltEditReplaceText()
{
	ReplaceDlg * pDlg = new ReplaceDlg( this, "replacetextdlg", /*bModal*/TRUE );;

	pDlg->adjustSize();

	// if any text is marked, use this text for searching
	if( m_pScript->hasMarkedTxt() )
	{
		pDlg->m_pFindText->setText( m_pScript->markedTxt() );
	}
	else
	{
		pDlg->m_pFindText->setText( m_sFindText );
	}
	pDlg->m_pReplaceText->setText( m_sReplaceText );
	pDlg->m_pRegularExpr->setChecked( m_bRegularExpr );
	pDlg->m_pCaseSensitive->setChecked( m_bCaseSensitive );
	pDlg->m_pOnlyWholeWord->setChecked( m_bOnlyWholeWords );
	pDlg->m_pSearchBackward->setChecked( m_bSearchBackward );
	pDlg->m_pReplaceAll->setChecked( m_bReplaceAll );

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_bIsReplace = true;

		m_sFindText = pDlg->m_pFindText->text();
		m_sReplaceText = pDlg->m_pReplaceText->text();
		m_bRegularExpr = pDlg->m_pRegularExpr->isChecked();
		m_bCaseSensitive = pDlg->m_pCaseSensitive->isChecked();
		m_bOnlyWholeWords = pDlg->m_pOnlyWholeWord->isChecked();
		m_bSearchBackward = pDlg->m_pSearchBackward->isChecked();
		m_bReplaceAll = pDlg->m_pReplaceAll->isChecked();

		bool bFound = m_pScript->findFirst( m_sFindText,
							  m_bRegularExpr,
		 					  m_bCaseSensitive,
							  m_bOnlyWholeWords,
							  !m_bSearchBackward );

		if( bFound )
		{
			m_pScript->replace( m_sReplaceText );

			while( m_bReplaceAll && bFound )
			{
				bFound = m_pScript->findNext();

				if( m_bIsReplace && bFound )
				{
					m_pScript->replace( m_sReplaceText );
				}
			}
		}
	}

	delete pDlg;
}

void VisiScript::sltClearScript()
{
	m_pScript->setText( "" );
    m_pScript->setEdited(FALSE);
}

#define MAX_BUFFER_LEN 512

void VisiScript::sltRun()
{
	QString sScript = m_pScript->text();

	if( !sScript.isNull() )
	{
		WriteTextFile( _TEMP_SCRIPT_FILE, (const char *)sScript );
	}

	char sCommand[MAX_BUFFER_LEN];
	const char * sScriptCmd = (const char *)m_sScriptCmd;

	QString sOutput;

	if( m_iExecuteMode == EXECUTE_MODE_POPEN )
	{
		sprintf( sCommand, "%s %s 2>%s", (sScriptCmd ? sScriptCmd : ""), _TEMP_SCRIPT_FILE, _TEMP_ERR_FILE );

		FILE * pPythonOutput = popen( sCommand, "r" );

		char sBuffer[MAX_BUFFER_LEN];
		size_t iSize = 0;
		do {
			iSize = fread( sBuffer, 1, MAX_BUFFER_LEN-1, pPythonOutput );
			sBuffer[iSize] = 0;
			if( iSize>0 )
			{
				sOutput += sBuffer;
			}
		} while( iSize>0 );

		pclose( pPythonOutput );
	}
	else
	{
		sprintf( sCommand, "%s %s 2>%s 1>%s", (sScriptCmd ? sScriptCmd : ""), _TEMP_SCRIPT_FILE, _TEMP_ERR_FILE, _TEMP_OUT_FILE );

		int iStatus;
		pid_t pid;

		pid = fork();
		if( pid == 0 )
		{
			execl( (const char *)m_sExeclCmd, (const char *)m_sExeclCmd, "--login", "-c", sCommand, 0 );
			exit( EXIT_FAILURE );
		}
		else if( pid < 0 )
		{
			/* fork failed ! */
			iStatus = -1;
			m_pOutput->setText( m_pOutput->text() + "Error forking process." );
		}
		else
		{
			if( waitpid( pid, &iStatus, 0 ) != pid )
			{
				iStatus = -1;
			}
		}
	}


	// show the output of the script in any case
	m_pOutput->setText( m_pOutput->text() + sOutput );

	// and also show the error-message, if available
	QString sErrText;
	bool bOk = ReadTextFile( _TEMP_ERR_FILE, sErrText );
	if( bOk && !sErrText.isNull() && sErrText.length()>0 )
	{
		sErrText = "Error: " + sErrText;
		m_pOutput->setText( m_pOutput->text() + sErrText );
	}
	remove( _TEMP_ERR_FILE );

	if( m_iExecuteMode == EXECUTE_MODE_EXECL )
	{
		// and also show the error-message, if available
		QString sOutText;
		bool bOk2 = ReadTextFile( _TEMP_OUT_FILE, sOutText );
		if( bOk2 && !sOutText.isNull() && sOutText.length()>0 )
		{
			m_pOutput->setText( m_pOutput->text() + sOutText );
		}
		remove( _TEMP_OUT_FILE );
	}

	m_pOutput->setCursorPosition( m_pOutput->numLines(), 1 );

	remove( _TEMP_SCRIPT_FILE );
}

bool VisiScript::CheckData()
{
	if( m_pScript->edited() )
	{
		int iRet = QMessageBox::information( this, tr( "Save file" ), tr( "File was modified, save now ?" ), QMessageBox::Yes, QMessageBox::No );

		if( iRet == QMessageBox::Yes )
		{
			sltFileSave();
		}
	}
	return true;
}

void VisiScript::sltNewFile( const DocLnk & f )
{
    DocLnk nf = f;
    nf.setType( /*"text/plain"*/m_sScriptExtention );	// Bugfix 30.8.2003
    nf.setName( "empty" );

	copyDocLnk( nf, m_aDoc );

    sltClearScript();

    sltShowScript();

    updateCaption( m_aDoc.name() );
}

void VisiScript::sltOpenFile( const DocLnk & f )
{
	// ACHTUNG: man MUSS eine deep-copy durchfuehren !!!
	//error:	m_aDoc = f;
	copyDocLnk( f, m_aDoc );

    sltClearScript();

    FileManager fm;
    QString txt;
    if ( !fm.loadFile( f, txt ) ) {
	// ####### could be a new file
	//qDebug( "Cannot open file" );
	//return;
    }

    m_pScript->setText( txt );
    m_pScript->setEdited(FALSE);

    sltShowScript();

    updateCaption( m_aDoc.name() );
}

void VisiScript::sltShowScript()
{
    m_pFileSelector->hide();
    m_pMenu->show();
    m_pButtonBar->show();
    m_pStack->raiseWidget( m_pVBox );
    m_pScript->setFocus();
}

void VisiScript::sltFileAboutToShow()
{
}

void VisiScript::sltEditAboutToShow()
{
	bool bEnableCopy = m_pScript->hasMarkedTxt() || m_pOutput->hasMarkedTxt();
	m_pCopyAction->setEnabled( bEnableCopy );
	m_pCutAction->setEnabled( bEnableCopy );

	QClipboard * pClipboard = QApplication::clipboard();
	QString sTemp = pClipboard->text();
	bool bEnablePaste = !sTemp.isNull();
	m_pPasteAction->setEnabled( bEnablePaste );
}

void VisiScript::sltViewAboutToShow()
{
	m_pToggleOverwrite->setOn( m_pScript->overwriteMode() );
}

void VisiScript::sltLanguagesAboutToShow()
{
	UpdateLanguageMenu();
}

void VisiScript::sltLanguageActivated( int iId )
{
	m_iLanguageID = iId;
	UpdateLanguage();
}

void VisiScript::updateCaption( const QString & sName=QString::null )
{
	if( sName.isNull() )
	{
		setCaption( tr( "VisiScript" ) );
	}
	else
	{
		setCaption( sName + " - " + tr( "VisiScript" ) );
	}
}

const char * c_sSection 		= "Script";
const char * c_sScriptCmdKey 	= "ScriptCmd";
const char * c_sExtentionKey	= "Extention";
const char * c_sFontNameKey 	= "FontName";
const char * c_sFontStyleKey 	= "FontStyle";
const char * c_sFontSizeKey 	= "FontSize";
const char * c_sWordWrapKey		= "WordWrap";
const char * c_sTabSpaceKey		= "TabSpace";
const char * c_sLanguageID		= "LanguageID";
const char * c_sOverwriteKey	= "Overwrite";
const char * c_sOutputRatio		= "OutputRatio";
const char * c_sLastDirectoryKey = "LastDirectory";
const char * c_sExeclCmdKey		= "ExeclCmd";
const char * c_sExecuteModeKey	= "ExecuteMode";
const char * c_sShowStaruptFileDialog	= "ShowFileDialog";
const char * c_sSpacesForTabs = "SpacesForTabs";
const char * c_sShowWhitespaces = "ShowWhitespaces";
const char * c_sShowLineNumbers= "ShowLineNumbers";
const char * c_sAutoIndent = "AutoIndent";

void VisiScript::loadInit()
{
	Config aCfg( _VISISCRIPT_NAME );

	m_sScriptExtention = GetExtentionFromScriptCmd( m_sScriptCmd );

	aCfg.setGroup( c_sSection );
	m_sScriptCmd = aCfg.readEntry( c_sScriptCmdKey, m_sScriptCmd );
	m_sScriptExtention = aCfg.readEntry( c_sExtentionKey, m_sScriptExtention );
	m_sFontName = aCfg.readEntry( c_sFontNameKey, m_sFontName );
	m_sFontStyle = aCfg.readEntry( c_sFontStyleKey, m_sFontStyle );
	m_iFontSize = aCfg.readNumEntry( c_sFontSizeKey, m_iFontSize );
	m_iWordWrap = aCfg.readNumEntry( c_sWordWrapKey, m_iWordWrap );
	m_iOverwrite = aCfg.readNumEntry( c_sOverwriteKey, m_iOverwrite );
	m_iTabSpace = aCfg.readNumEntry( c_sTabSpaceKey, m_iTabSpace );
	m_iLanguageID = aCfg.readNumEntry( c_sLanguageID, m_iLanguageID );
	m_iOutputRatio = aCfg.readNumEntry( c_sOutputRatio, m_iOutputRatio );
	m_iSpacesForTabs = aCfg.readNumEntry( c_sSpacesForTabs, m_iSpacesForTabs );
	m_iShowWhitespaces = aCfg.readNumEntry( c_sShowWhitespaces, m_iShowWhitespaces );
	m_iShowLineNumbers = aCfg.readNumEntry( c_sShowLineNumbers, m_iShowLineNumbers );
	m_iAutoIndent = aCfg.readNumEntry( c_sAutoIndent, m_iAutoIndent );
	m_sLastDirectory = aCfg.readEntry( c_sLastDirectoryKey, m_sLastDirectory );
	m_sExeclCmd = aCfg.readEntry( c_sExeclCmdKey, m_sExeclCmd );
	m_iExecuteMode = aCfg.readNumEntry( c_sExecuteModeKey, m_iExecuteMode );
	m_iShowStartupFileDialog = aCfg.readNumEntry( c_sShowStaruptFileDialog, m_iShowStartupFileDialog );
}

void VisiScript::saveInit()
{
	Config aCfg( _VISISCRIPT_NAME );

	aCfg.setGroup( c_sSection );
	aCfg.writeEntry( c_sScriptCmdKey, m_sScriptCmd );
	aCfg.writeEntry( c_sExtentionKey, m_sScriptExtention );
	aCfg.writeEntry( c_sFontNameKey, m_sFontName );
	aCfg.writeEntry( c_sFontStyleKey, m_sFontStyle );
	aCfg.writeEntry( c_sFontSizeKey, m_iFontSize );
	aCfg.writeEntry( c_sWordWrapKey, m_iWordWrap );
	aCfg.writeEntry( c_sOverwriteKey, (int)m_pScript->overwriteMode()/*m_iOverwrite*/ );
	aCfg.writeEntry( c_sTabSpaceKey, m_iTabSpace );
	aCfg.writeEntry( c_sLanguageID, m_iLanguageID );
	aCfg.writeEntry( c_sOutputRatio, m_iOutputRatio );
	aCfg.writeEntry( c_sSpacesForTabs, m_iSpacesForTabs );
	aCfg.writeEntry( c_sShowWhitespaces, m_iShowWhitespaces );
	aCfg.writeEntry( c_sShowLineNumbers, m_iShowLineNumbers );
	aCfg.writeEntry( c_sAutoIndent, m_iAutoIndent );
	aCfg.writeEntry( c_sLastDirectoryKey, m_sLastDirectory );
	aCfg.writeEntry( c_sExeclCmdKey, m_sExeclCmd );
	aCfg.writeEntry( c_sExecuteModeKey, m_iExecuteMode );
	aCfg.writeEntry( c_sShowStaruptFileDialog, m_iShowStartupFileDialog );
}

void VisiScript::closeEvent( QCloseEvent * pCloseEvent )
{
	CheckData();

	saveInit();

	pCloseEvent->accept();
}

