/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source$
 *
 *  $Revision$
 *
 *	$Log$
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002 by Michael Neuroth.										 *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

#ifndef VISISCRIPT_H
#define VISISCRIPT_H

#include <qpe/filemanager.h>
#include <qpe/applnk.h>

#include <qmainwindow.h>

// Problems with save-as functionality with opie file-dialog
//#define _with_opie_filedialog

class FileSelector;
class QToolBar;
class QPEMenuBar;
class QWidgetStack;
class QVBox;
class QMultiLineEdit;
class QTextView;
class QPopupMenu;
class QAction;

class MyQMultiLineEdit;		// to invoke the method hasMarkedText()
class MyQCodeEdit;
class FileDialogS;
class QextScintillaLexer;

// ************************************************************************

class VisiScript : public QMainWindow
{
    Q_OBJECT

public:
    VisiScript( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~VisiScript();

public slots:
    void sltFileNew();
    void sltFileOpen();
    void sltFileSave();
    void sltFileSaveAs();
    void sltFileSaveOutputAs();
    void sltFileImport();
    void sltFileExport();
    void sltEditUndo();
    void sltEditRedo();
    void sltEditCut();
    void sltEditCopy();
    void sltEditPaste();
    void sltEditFindText();
    void sltEditFindNext();
    void sltEditReplaceText();
    void sltClearOutput();
    void sltClearScript();
    void sltRun();
    void sltSelectFont();
	//void sltSelectHelvetica();
	//void sltSelectFixed();
	//void sltSelectUnifont();
	//void sltSelectSmall();
	//void sltSelectMedium();
	//void sltSelectLarge();
	void sltZoomIn();
	void sltZoomOut();
	void sltMaximizeOutput();
	void sltMaximizeScript();
	void sltToggleWordWrap();
	void sltToggleOverwrite();
    void sltShowAbout();

private slots:
    void sltNewFile( const DocLnk & );
    void sltOpenFile( const DocLnk & );
    //void sltOpenFile( const QString & );
    void sltConfiguration();
    void sltShowScript();
    void sltFileAboutToShow();
    void sltEditAboutToShow();
    void sltViewAboutToShow();
    void sltLanguagesAboutToShow();
    void sltLanguageActivated( int iId );
#ifndef _with_opie_filedialog
    void sltDoImport();
    void sltDoExport();
#endif

protected:
    void closeEvent( QCloseEvent * pCloseEvent );

private:
    void updateCaption( const QString & sName=QString::null );
    void updateFileSelector();
    void loadInit();
    void saveInit();
    bool CheckData();
    void UpdateFont();
    void UpdateFlags();
    void UpdateLanguage();
    void UpdateLanguageMenu();
    void UpdateScriptOutputRatio();
    int GetHeightForOutput( int iPercent );

	// ** data **
#ifndef _with_opie_filedialog
	FileDialogS *		m_pExportFileDlg;
#endif

	FileSelector *		m_pFileSelector;
	QToolBar *			m_pMenu;
	QToolBar *			m_pButtonBar;
	QPEMenuBar *		m_pMenuBar;
	QWidgetStack *		m_pStack;
	QVBox * 			m_pVBox;
	MyQCodeEdit * 		m_pScript;
	QextScintillaLexer * m_pLexer;
	//QTextView *			m_pScript;		// only readonly possible !
	MyQMultiLineEdit *	m_pOutput;

    QPopupMenu * 		pFile;
    QPopupMenu * 		pEdit;
	QPopupMenu * 		pView;
	QPopupMenu *		pLanguages;

	//QAction *			m_pFixedAction;
	//QAction *			m_pHelveticaAction;
	//QAction *			m_pUnifontAction;
	//QAction *			m_pSmallAction;
	//QAction *			m_pMediumAction;
	//QAction *			m_pLargeAction;
	QAction *			m_pMaximizeScript;
	QAction *			m_pMaximizeOutput;
	QAction *			m_pToggleWordWrap;
	QAction *			m_pToggleOverwrite;
	QAction *			m_pCopyAction;
	QAction *			m_pCutAction;
	QAction *			m_pPasteAction;

    DocLnk 				m_aDoc;

	QString				m_sScriptCmd;
	QString				m_sScriptExtention;

	QString				m_sFontName;
	QString				m_sFontStyle;
	int					m_iFontSize;

	bool				m_bSaveOutput;

	int					m_iWordWrap;
	int					m_iOverwrite;		// temp
	int					m_iSpacesForTabs;
	int					m_iShowWhitespaces;
	int					m_iShowLineNumbers;
	int					m_iAutoIndent;
	int					m_iTabSpace;
	int					m_iLanguageID;
	QString				m_sLastDirectory;
	QString				m_sExeclCmd;
	int					m_iExecuteMode;		// 1 == popen, 2 == fork/execl
	int					m_iShowStartupFileDialog;
	int					m_iOutputRatio;		// for the ratio of Script and output
	QString				m_sFindText;		// temps for find/replace dialog
	QString				m_sReplaceText;
	bool				m_bIsReplace;
	bool				m_bRegularExpr;
	bool				m_bCaseSensitive;
	bool				m_bOnlyWholeWords;
	bool				m_bSearchBackward;
	bool				m_bReplaceAll;
};

const int EXECUTE_MODE_POPEN = 1;
const int EXECUTE_MODE_EXECL = 2;

#endif // VISISCRIPT
